/* 
   
   Geeks Artificial Neural Network (G.A.N.N) - Neural Map Simulator
   
   Copyright GNU/GPL 2006 George Delaportas (qtgeo)
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

*/



//Geeks Neural Map Simulator (0:Learn,1:Run)
void GNMS(int SimFunc)

{

    //Variables
    register int i,j,k,n,p;	//Counters
    int GNFT;			//GNeuron Function Type
    int Offset;			//An Offset
    int Tracer;			//A Tracer
    float NData[MAX_PIN];	//New Data From Inputs GSocket
    float GData[MAX_PIN];	//Data Produced By G.A.N.N
    float ErrStat;		//Error Statistics (%)
    float Tmp;			//Temp Values
    
    //Check For Wrong Simulation Mode
    if (SimFunc<0 || SimFunc>2)
    
    {
    
        printf("Error: Wrong Simulation Mode Specified!\n");
        
        exit(1);
    
    }
    
    //Create A New G.A.N.N In Memory (G.A.N.N Matrix)
    GNeuron *GANN;
    GANN = new GNeuron[GNNum];
    
    //Get I/O From GSockets Loaded In Memory
    Inputs=InSocket.GetPins();
    Outputs=OutSocket.GetPins();
    
    //Fill NData With Inputs GSocket Data (Only For Uniform Checks)
    for (i=0;i<Inputs;i++)
        NData[i]=InSocket.GetPV(i,0);
    
    //Check GMap Simulation Type So As To Set GNeurons Function Type
    if (GMSType==0)
        GNFT=0;
    else
        GNFT=2;
    
    //Learn
    if (SimFunc==0)
    
    {
    
        //Print Inputs GSocket
        printf("----- Inputs GSocket -----\n");   
        
        for (i=0;i<Inputs;i++)
        
        {
        
            printf("GPin:%i\n",i+1);
            printf("Minimum Value:%.1f\n",InSocket.GetPV(i,0));
            
            if (InSocket.GetPV(i,1)==0.0)
                printf("Maximum Value:N/A\n");
            else
                printf("Maximum Value:%.1f\n",InSocket.GetPV(i,1));
        
        }
        
        printf("--------------------------\n\n");
        
        printf("Start Learning...\n\n");
        
        //Simulations Loop
        for (n=0;n<SimNum;n++)
        
        {
        
            //Escape Caption
            ReRoute:;
            
            //Reset Tracer
            Tracer=0;
            
            //Reset Offest
            Offset=0;
            
            //Start Routing Through The Layers
            for (i=0;i<Layers;i++)
            
            {
            
                //Get GNeurons Of Current Layer
                k=GMap[i];
                
                //Check GMap Connections Type (0:Auto,1:Manual)
                if (GMCType==0)
                
                {
                
                    //Enable All GNeurons In This Layer
                    for (j=Offset;j<(k+Offset);j++)
                    
                    {
                    
                        //Get Data From InSocket (First Layer)
                        if (i==0)
                        
                        {
                        
                            for (p=0;p<Inputs;p++)
                            
                            {
                            
                                //Get Data Or Data Range
                                if (InSocket.GetPV(p,1)==0.0)
                                    Tmp=InSocket.GetPV(p,0);
                                else
                                    Tmp=GGenData(InSocket.GetPV(p,0),\
                                    InSocket.GetPV(p,1));
                                
                                //Give Data To This GNeuron
                                GANN[j].InData(p,Tmp);
                            
                            }
                            
                            //Enable This GNeuron
                            GANN[j].Enable(GNFT);
                            
                            printf("Neuron %i Enabled!\n\n",j+1);
                        
                        }
                        
                        //Get Data From Previous Layer
                        else
                        
                        {
                        
                            for (p=0;p<Offset;p++)
                            
                            {
                            
                                //Give Data To This GNeuron
                                GANN[j].InData(p,GANN[p].OutData());
                            
                            }
                            
                            //Enable This GNeuron
                            GANN[j].Enable(GNFT);
                            
                            printf("Neuron %i Enabled!\n\n",j+1);
                        
                        }
                    
                    }
                    
                    //Set New Offset
                    if (i<Layers-1)
                        Offset=k;
                
                }
                
                else
                
                {
                
                    //User Defined Connections
                    
                
                }
            
            }
            
            //Reset j
            j=0;
            
            //Fill GData
            for (i=Offset;i<(k+Offset);i++)
            
            {
            
                GData[j]=GANN[i].OutData();
                
                j++;
            
            }
            
            //Gather Error Statistics (0:Uniform,1:Non-Uniform)
            ErrStat=GStat(NData,GData,ECT);
            
            //Check Error Statistics
            if (ErrStat>=Threshold)
            
            {
            
                //Save This G.A.N.N To The GKDB
                GSaveData(GANN);
                
                //Print Outputs GSocket
                printf("----- Outputs GSocket -----\n");
                
                for (i=0;i<Outputs;i++)
                
                {
                
                    printf("GPin:%i\n",i+1);
                    printf("Value:%.1f\n",GData[i]);
                
                }
                
                printf("---------------------------\n\n");
                
                printf("Learning Successfully Completed!\n");
            
            }
            
            else
            
            {
            
                //Error Correction
                for (i=0;i<Outputs;i++)
                
                {
                
                    if (GData[i]<OutSocket.GetPV(i,0))
                        GErrC(GANN,i+Offset,ErrStat,0);
                    
                    if (OutSocket.GetPV(i,1)!=0.0 &&\
                    GData[i]>OutSocket.GetPV(i,1))
                        GErrC(GANN,i+Offset,ErrStat,0);
                
                }
                
                goto ReRoute;
            
            }
        
        }
    
    }
    
    //Run
    else
    
    {
    
        
    
    }
    
    //UnLoad G.A.N.N From Memory (G.A.N.N Matrix)
    delete [] GANN;

}

/* ------------------------------------------------------------------------ */
